# -*- coding: utf-8 -*-
import uuid
import logging
from gisweb_ads.schema import IProtocolloResult
from bs4 import BeautifulSoup

_logger = logging.getLogger('gisweb.sicraweb')


def parseMessage(xmlMessage) -> IProtocolloResult:
    # memo xml tiene conto delle maiuscole lxml tutto minuscolo
    soup = BeautifulSoup(xmlMessage, 'xml')
    keys = ["lngDocID", "lngErrNumber", "strErrString",
            "lngNumPG", "lngAnnoPG", "strDataPG"]
    dz = dict()

    for key in keys:
        dz[key] = soup.find(key) and soup.find(key).string  # type: ignore

    return IProtocolloResult(
        lngDocID=int(dz.get('lngDocID') or 0),
        lngNumPG=int(dz.get('lngNumPG') or 0),
        lngAnnoPG=int(dz.get('lngAnnoPG') or 0),
        lngErrNumber=int(dz.get('lngErrNumber') or 0),
        strErrString=dz.get('strErrString') or '',
        strDataPG=dz.get('strDataPG') or '',
    )


def with_soap_attachment(suds_method, attachment_data, *args, **kwargs) -> IProtocolloResult:
    """ Add an attachment to a suds soap request.

    attachment_data is assumed to contain a list:
      ( <attachment content>, <content id>, <mime-type> )

    The attachment content is only required required list element.

    http://stackoverflow.com/questions/17046779/passing-file-attachments-in-soap-using-suds

    """

    from suds.transport import Request

    # Suds doesn't currently support SOAP Attachments, so we have to build our
    # own attachment support, using parts of the suds library

    MIME_DEFAULT = 'text/plain'
    attachment_encoding = '8bit'
    attachment_mimetype = MIME_DEFAULT
    attachment_id = ''
    soap_method = suds_method.method
    data: bytes

    if len(attachment_data) == 3:
        data, attachment_id, attachment_mimetype = attachment_data
    elif len(attachment_data) == 2:
        data, attachment_id = attachment_data
    elif len(attachment_data) == 1:
        data = attachment_data
        attachment_id = uuid.uuid4()

    # Generate SOAP XML appropriate for this request
    binding = soap_method.binding.input
    soap_xml = binding.get_message(
        soap_method, args, kwargs).str().encode('utf-8')

    # Prepare MIME headers & boundaries
    boundary_id = f'uuid:{uuid.uuid4()}'
    root_part_id = f'uuid:{uuid.uuid4()}'
    request_headers = {
        'Content-Type': '; '.join([
            'multipart/related',
            'type="text/xml"',
            f'start="<{root_part_id}>"',
            f'boundary="{boundary_id}"',
        ]),
        'SOAPAction': ''
    }
    soap_headers = '\n'.join([
        'Content-Type: text/xml; charset=UTF-8',
        'Content-Transfer-Encoding: 8bit',
        f'Content-Id: <{root_part_id}>',
        '',
    ]).encode('utf-8')
    attachment_headers = '\n'.join([
        f'Content-Type: {attachment_mimetype}',
        f'Content-Transfer-Encoding: {attachment_encoding}',
        f'Content-Id: <{attachment_id}>',
        '',
    ]).encode('utf-8')

    boundary = f"--{boundary_id}".encode('utf-8')

    # Build the full request
    request_text = b'\r\n'.join([
        boundary,
        soap_headers,
        soap_xml,
        boundary,
        attachment_headers,
        data,
        boundary + '--'.encode('utf-8')
    ])

    # with open("./test.xml", "w") as f:
    #  f.write(request_text.decode('latin-1'))

    # Stuff everything into a request object
    headers = suds_method.client.options.headers.copy()
    headers.update(request_headers)
    request = Request(suds_method.client.wsdl.url, request_text)
    request.headers = headers
    # Send the request
    # _logger.info(request_text)

    response = suds_method.client.options.transport.send(request)

    if response.code == 200:
        try:
            return parseMessage(response.message)
        except Exception as e:
            with open("./errori_protocollo.txt", "a") as f:
                f.write(str(response.message))
            return IProtocolloResult(lngErrNumber=999, strErrString=str(response.message))
    else:
        return IProtocolloResult(lngErrNumber=999, strErrString="ERRORE NELLA RISPOSTA DEL SERVIZIO")



from typing import Any
from pydantic import BaseModel, EmailStr
from enum import Enum

class IFlussoEnum(str, Enum):
    ENTRATA = "E"
    USCITA = "U"
    INTERNO = "I"

class IAzioneEnum(str, Enum):
    ESEGUI = "ESEGUI"
    CARICO = "CARICO"
    
class ISmistamentoEnum(str, Enum):
    CONOSCENZA = "CONOSCENZA"
    COMPETENZA = "COMPETENZA"

class IAmministrazione(BaseModel):
    Denominazione: str
    CodiceAOO: str
    CodiceEnte: str
    IndirizzoTelematico: EmailStr

class IConfigProtocollo(BaseModel):
    wsUrl: str
    wsUser: str
    wsEnte: str
    wsPassword: str
    amministrazione: IAmministrazione
    applicativo: str | None = None
    FAKE: bool = False
    
class BaseRet(BaseModel):
    lngErrNumber: int = 0
    strErrString: str = ''

class ILoginRet(BaseRet):
    strDST: str | None
    
class IProtocolloResult(BaseRet):
    lngNumPG: int = 0
    lngAnnoPG: int = 0
    strDataPG: str = ''
    lngDocID: int = 0


class IAllegato(BaseModel):
    id: int | None = None
    descrizione: str
    tipo: str
    nome: str
    content: Any
    size: int
    mimetype: str
    ext: str

class IFascicolo(BaseModel):
    numero: str = ""
    anno:str = ""
    
class IParametro(BaseModel):
    nome: str
    valore: str

class ISoggettoProtocollo(BaseModel):
    Nome: str | None = None
    Cognome: str | None = None
    Denominazione: str | None = None
    CodiceFiscale: str
    IndirizzoTelematico: str
    TipoSoggetto: str | None = ""
    Principale: bool = True
    Titolo: str | None = ""
    
class IDataProtocollo(BaseModel):
    Soggetti: list[ISoggettoProtocollo] = []
    Flusso: str = "E"
    Oggetto: str = "OGGETTO ISTANZA"
    Classifica: str = "06-01"
    UO: str = "2.5.1"
    Fascicolo: IFascicolo | None = None
    Parametri: list[IParametro] | None = []
    TipoDocumento: str = "WSTS"
    Principale: IAllegato | str = "documento_riepilogo"
    Allegati: list[IAllegato] | list[str] = []
    




from ast import Dict
import random
from typing import Any
from gisweb_ads.schema import IAllegato, IConfigProtocollo, ILoginRet, IProtocolloResult, IDataProtocollo
from gisweb_ads.soap_attachments import with_soap_attachment
from suds.client import Client
import logging
import os, base64, uuid
import json
from datetime import datetime
from pydantic import BaseModel
from jinja2 import Environment, PackageLoader, select_autoescape
import time

_logger = logging.getLogger('gisweb-ads')

env = Environment(
    loader=PackageLoader("gisweb_ads"),
    autoescape=select_autoescape()
)

class Protocollo:

    config: IConfigProtocollo
    wsClient:Any
    DST:str | None = None
    wsError: str
    dataProt: IDataProtocollo
    
    def __init__(self, config:IConfigProtocollo):
        self.config = config
        try:
            self.wsClient = Client(config.wsUrl)
            result:ILoginRet = self.wsClient.service.login(config.wsEnte, config.wsUser, config.wsPassword)
            if result.lngErrNumber!=0:
                self.wsError = result.strErrString
            else:
                self.DST = result.strDST
                
        except Exception as error:
            self.wsError = str(error)

    
    def __inserisciAllegato(self, allegato:IAllegato):

        try:
            fName = allegato.nome#.encode('ascii','ignore')
            mime = allegato.mimetype#.encode('ascii','ignore')
            content = allegato.content
            result = with_soap_attachment(self.wsClient.service.inserimento, [content,fName,mime], self.config.wsUser, self.DST)
        except Exception as error:
            _logger.info(error)
            return error
             
        if result.lngErrNumber != 0:
            _logger.info(result.strErrString)
            return result.strErrString
        
        allegato.content = None
        allegato.id = result.lngDocID
        

    def __getSegnatura(self) -> str:
        
        #inserisco allegato principale
        if isinstance(self.dataProt.Principale, IAllegato):
            error = self.__inserisciAllegato(self.dataProt.Principale)
            if error:
                return ""
   
        #inserisco altri allegati
        for documento in self.dataProt.Allegati:
            if isinstance(documento, IAllegato):
                error = self.__inserisciAllegato(documento)
                if error:
                    return "" 

        template = env.get_template("segnatura.xml")
        context = self.dataProt.model_copy(update=dict(
            Today = datetime.today().strftime('%d/%m/%Y'),
            totAllegati = len(self.dataProt.Allegati),
            Amministrazione = self.config.amministrazione,
            Applicativo = self.config.applicativo
        ))
        
        return template.render(context)


    def protocollaDocumento(self, data:IDataProtocollo, testXml:bool=True) -> str | IProtocolloResult:
                        
        if not self.DST:
            return IProtocolloResult(lngErrNumber=999, strErrString= str(self.wsError))
                
        self.dataProt = data
        xmlSegnatura =  self.__getSegnatura()
        if testXml:
            return xmlSegnatura
        
        if self.config.FAKE:
            time.sleep(15)
            #today = datetime.now().isoformat()
            #today = datetime.now().strftime('%d/%m/%Y') 
            anno = 9999
            numero = random.randint(100,999)
            return IProtocolloResult(lngNumPG=numero, lngAnnoPG=int(anno), lngDocID=0)
        

        #Protocollazione Documento
        try:
            result = with_soap_attachment(self.wsClient.service.protocollazione,
                                          [xmlSegnatura.encode('utf-8'), 'profilazione', 'text/xml'], self.config.wsUser, self.DST)  
            
            if result.strErrString:
                with open("./errori_protocollo.txt", "a") as f:
                    f.write(result.strErrString)
                
            return result
            
        except Exception as e:
            with open("./errori_protocollo.xml", "a") as f:
                f.write(f"{data.model_dump()} {str(e)}")
            return IProtocolloResult(lngErrNumber=999, strErrString= str(e))
        

        # se protocollazione in uscita aggiungo la segnatura e oggetto
        ''''
        if flusso=='U':
            self.numero_protocollo="%07d" %result['lngNumPG']
            self.data_protocollo=datetime.today().strftime("%d/%m/%Y")
            # Segnatura di protocollazione
            tplSegnatura = ViewPageTemplateFile("templates/segnatura_pec.pt")
            xmlSegnatura = tplSegnatura(self)
        '''

